'==========================================================================
'
' VBScript Source File
'
' NAME: CreateSendToEntry.vbs
'
' AUTHOR: Philippe BEAUREILLES
' DATE  : 2014/12/10
'
' COMMENT: Creates a Send To entry for Qccrypt
'
'==========================================================================
Option Explicit


Const ciOpenFileForReading_G = 1
Const ciOpenFileForWriting_G = 2
Const ciOpenFileForAppending_G = 8

Dim oFS_G : Set oFS_G = CreateObject("Scripting.FileSystemObject")
Dim oWS_G : Set oWS_G = WScript.CreateObject("WScript.Shell")
Dim oWN_G : Set oWN_G = CreateObject("Wscript.Network")
Dim strPackagePath_G:strPackagePath_G = oFS_G.GetParentFolderName(WScript.ScriptFullName)
Dim strTempFolder_G:strTempFolder_G = oFS_G.GetSpecialFolder(2)
Dim strLocalLogFile_G : strLocalLogFile_G =  oFS_G.BuildPath(strTempFolder_G, WScript.ScriptName & ".log")
Dim dvReturnCodes_G : Set dvReturnCodes_G = CreateObject("Scripting.Dictionary")
Dim blnLocalLogFileExists_G : blnLocalLogFileExists_G = False
Public oLogTxt_G

Dim strQccryptAppPathRegKey_G:strQccryptAppPathRegKey_G = "HKLM\Software\Microsoft\Windows\CurrentVersion\App Paths\Qccrypt.exe\Path"
Dim strSendToShortcutFullName_G:strSendToShortcutFullName_G = oFS_G.BuildPath(oWS_G.SpecialFolders("SendTo"),"Qccrypt.lnk")
Dim strQccryptPath_G, strQccryptExeFullName_G
Dim oSendToShortcut_G

'---------------------------------------------------------------------------
'Initializing return codes
dvReturnCodes_G.Add "OK", 0
dvReturnCodes_G.Add "ALREADY_DONE", 0
dvReturnCodes_G.Add "APPPATH_NOT_FOUND", 0
dvReturnCodes_G.Add "QCCRYPT_EXE_NOT_FOUND", 0
dvReturnCodes_G.Add "CREATE_SHORTCUT_ERROR", 1

'---------------------------------------------------------------------------
'Initializing log file
Set oLogTxt_G = New LogObject
oLogTxt_G.strLogFileName = strLocalLogFile_G
CreateFolderAndParentFolders oFS_G.GetParentFolderName(strLocalLogFile_G)
If Not oLogTxt_G.StartLog() Then
	DisplayConsole "Unable to create """ & oLogTxt_G.strLogFileName & """ log file name!"
	TerminateScript "CREATE_LOG_FILE_ERROR"
Else
	blnLocalLogFileExists_G = True
End If

'---------------------------------------------------------------------------
'Check if the Send To shortcut of Qccrypt does already exist
oLogTxt_G.AddStep "Check if the Send To shortcut of Qccrypt does already exist"
If oFS_G.FileExists(strSendToShortcutFullName_G) Then
	'Already done, exit.
	oLogTxt_G.AddLog "The shortcut does already exist. Nothing to do."
	TerminateScript "ALREADY_DONE"
Else
	oLogTxt_G.AddLog "Shortcut not found."
End If

'---------------------------------------------------------------------------
'Search for Qccrypt application
oLogTxt_G.AddStep "Search for Qccrypt application"
oLogTxt_G.AddLog "Reading """ & strQccryptAppPathRegKey_G & """ registry key..."
On Error Resume Next
Err.Clear
strQccryptPath_G = oWS_G.RegRead(strQccryptAppPathRegKey_G)
If Err.Number <> 0 Then
	'Qccrypt Path Not Found. Exit.
	oLogTxt_G.AddLog "Registry key not found. The application seems to be not or bad installed."
	TerminateScript "APPPATH_NOT_FOUND"
Else
	oLogTxt_G.AddLog "Path found: " & strQccryptPath_G
End If
On Error GoTo 0
strQccryptExeFullName_G = oFS_G.BuildPath(strQccryptPath_G,"Qccrypt.exe")
oLogTxt_G.AddLog "Searching for """ & strQccryptExeFullName_G & """ file..."
If Not oFS_G.FileExists(strQccryptExeFullName_G) Then
	'Qccrypt binary not found. Exit
	oLogTxt_G.AddLog "File not found!  The application seems to be not or bad installed."
	TerminateScript "QCCRYPT_EXE_NOT_FOUND"
Else
	oLogTxt_G.AddLog "File found."
End If

'---------------------------------------------------------------------------
'Create the SendTo shortcut
oLogTxt_G.AddStep "Create the SendTo shortcut"
On Error Resume Next
Err.Clear
Set oSendToShortcut_G = oWS_G.CreateShortcut(strSendToShortcutFullName_G)
oSendToShortcut_G.TargetPath = strQccryptExeFullName_G
oSendToShortcut_G.IconLocation = strQccryptExeFullName_G & ", 0"
oSendToShortcut_G.Save
If Err.Number <> 0 Then
	oLogTxt_G.AddLog "An error was reported when creating the  shortcut. Error Description: " & Err.Description
	TerminateScript "CREATE_SHORTCUT_ERROR"
Else
	oLogTxt_G.AddLog "Send To shortcut created successfully!"
End If
On Error GoTo 0

'---------------------------------------------------------------------------
'End of job
TerminateScript "OK"

'**************************************************************************
' Classes, Subs and Functions
'**************************************************************************
Sub DisplayConsole(strTextToDisplay)
	If InStr(UCase(WScript.FullName), "CSCRIPT.EXE") <> 0 Then
		WScript.Echo strTextToDisplay
	End If
End Sub

'---------------------------------------------------------------------------
Class LogObject
	
	Public strLogFileName
	Private oLogFile
	Private strSeparationCharCount

	Private Sub Class_Initialize()
		strSeparationCharCount = 140
	End Sub
	
	Public Function StartLog()
		On Error Resume Next
		Err.Clear
		Set oLogFile = oFS_G.OpenTextFile(strLogFileName, 8, True)
		If Err <> 0 Then
			DisplayConsole "An error occured while opening """ & strLogFileName & """ log file in write mode."
			DisplayConsole "Error description: " & Err.Description
			Err.Clear
			On Error GoTo 0
			StartLog = False
			Exit Function
		Else
			StartLog = True
			AddLog VbCrLf & String(strSeparationCharCount, "#") & VbCrLf & "# " & CStr(Now()) & " - Begin log" & VbCrLf & String(strSeparationCharCount, "#") & VbCrLf
		End If
		On Error GoTo 0
	End Function
	
	Public Sub AddLog(strLogText)
		DisplayConsole strLogText
		On Error Resume Next
		oLogFile.WriteLine(strLogText)
		On Error GoTo 0
	End Sub

	Public Sub AddStep(strLogText)
		strLogText = String(strSeparationCharCount, "-") & VbCrLf & "| " & CStr(Now()) & " - " & strLogText & VbCrLf & String(strSeparationCharCount, "-")
		DisplayConsole strLogText
		On Error Resume Next
		oLogFile.WriteLine(strLogText)
		On Error GoTo 0
	End Sub

	Private Sub DisplayConsole(strTextToDisplay)
		If InStr(UCase(WScript.FullName), "CSCRIPT.EXE") <> 0 Then
			WScript.Echo strTextToDisplay
		End If
	End Sub

	Private Sub Class_Terminate()
		AddLog VbCrLf
		AddLog VbCrLf & String(strSeparationCharCount, "#") & VbCrLf & "# " & CStr(Now()) & " - End log" & VbCrLf & String(strSeparationCharCount, "#")
		On Error Resume Next
		oLogFile.Close()
		Err.Clear
		On Error GoTo 0
	End Sub
	
End Class

'---------------------------------------------------------------------------
Sub TerminateScript(strRetCode)
	Dim strMsg
	strMsg =  "Return code: " & dvReturnCodes_G.Item(strRetCode) & VbCrLf & "Return status: " & strRetCode
	If blnLocalLogFileExists_G Then
		oLogTxt_G.AddStep "End of script"
		oLogTxt_G.AddLog strMsg
	Else
		DisplayConsole "End of script"
		DisplayConsole strMsg
	End If
	WScript.Quit(dvReturnCodes_G.Item(strRetCode))
End Sub

'---------------------------------------------------------------------------
Function FilledString(strChaine, iNbFinal, strChar)
	FilledString = Right(String(iNbFinal, strChar)&strChaine, iNbFinal)
End Function

'---------------------------------------------------------------------------
'Create the folder "strFolderPathName" and all its parent's folders
Sub CreateFolderAndParentFolders(strFolderPathName)
	If Not oFS_G.FolderExists(strFolderPathName) Then
		Dim strCurrentFolder, arrFolders(), oFolder, strRootFolder
		Set oFolder = oFS_G.GetDrive(oFS_G.GetDriveName(oFS_G.GetAbsolutePathName(strFolderPathName)))
		strRootFolder = oFolder.RootFolder
		strCurrentFolder = strFolderPathName
		ReDim arrFolders(-1)
		While strCurrentFolder <> strRootFolder
			ReDim Preserve arrFolders(UBound(arrFolders)+1)
			arrFolders(UBound(arrFolders)) = strCurrentFolder
			strCurrentFolder = oFS_G.GetParentFolderName(strCurrentFolder)
		Wend
		Dim i
		For i = UBound(arrFolders) To 0 Step -1
			If Not oFS_G.FolderExists(arrFolders(i)) Then
				On Error Resume Next
				oFS_G.CreateFolder arrFolders(i)
				If Err <> 0 Then
					oLogTxt_G.AddLog "An error has occurred during the creation of """ & arrFolders(i) & """ folder. Error description : " & Err.Description
					Err.Clear
				Else
					oLogTxt_G.AddLog """" & arrFolders(i) & """ folder created successfully."
				End If
				On Error GoTo 0
			End If
		Next
	End If
End Sub


